unit FileFinder;

{ This component is a file search component here some key knowledge about this
	component.
  1. This is a thread like component, which means it will allow other objects
     to do processes, but it won't stop if you shutdown your application, so
     put FileFinder.Stop under the Form.Close event if necessary.
  2. I used orignal TSearchRec for the callback parameter, you can find it in
  	 dephi's help files.
  3. If the serach is stopped then it will call the OnStop event.
  4. You can stop the search after you found your file.
	   put Stop := True(default = False) under the OnFileFound Event.
  5. Property Stopped is readonly, you set it stop by call method Stop.

  What is New
  1. StatusLabel Link: Display the directory name while searching.
  2. FuzzySearch.

  Author Info.
  William Yang, yang@btinternet.com
  http://www.btinternet.com/~yang/
}

interface

uses Windows, SysUtils, Forms, Messages, Classes, StdCtrls, StrUtils;

type
	TFileFoundEvent = procedure (Sender: TObject; Directory: String;
  	SearchRec: TSearchRec; var Stop : Boolean) of object;

  TFileFinder = class(TComponent)
  private
    { Private Declarations }
    FFileAttr: Integer;
    FRoot: String;
    FStop: Boolean;
    FOnFound: TFileFoundEvent;
    FFuzzySearch: Boolean;
    FSubFolders: Boolean;
    FProcessMsgs: Boolean;
    FOnStop: TNotifyEvent;
    FLevel : Integer;
    FFilename: String;
    FIgnoreCase: Boolean;
    FStatus: TLabel;
  protected
    { Protected Declarations }
		procedure GetSubDirs(ARoot: String);
  public
    { Public Declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Search;
    procedure Stop;    
  published
    property FileAttr: Integer read FFileAttr write FFileAttr;
    property Root: String read FRoot write FRoot;
    property SubFolders: Boolean read FSubFolders write FSubFolders;
    property ProcessMessages: Boolean read FProcessMsgs write FProcessMsgs;
    property Filename: String read FFilename write FFilename;
    property StatusLabel: TLabel read FStatus write FStatus;
    property FuzzySearch: Boolean read FFuzzySearch write FFuzzySearch;
    property IgnoreCase: Boolean read FIgnoreCase write FIgnoreCase;
    property Stopped : Boolean read FStop;
    property OnFileFound: TFileFoundEvent read FOnFound write FOnFound;
    property OnStop: TNotifyEvent read FOnStop write FOnStop;
  end;

procedure Register;

implementation

{--------------------------------------}
{ TFileFinder.Create                   }
{--------------------------------------}
constructor TFileFinder.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FRoot := 'C:\';
  FStop := False;
  FSubFolders := True;
  FFileAttr := faAnyFile;
	FProcessMsgs := True;
  FLevel := 0;
  FIgnoreCase := True;
	FFuzzySearch := False;
	FFilename := '';
end;

{--------------------------------------}
{ TFileFinder.Destroy                  }
{--------------------------------------}
destructor TFileFinder.Destroy;
begin
  inherited Destroy;
  if Assigned(FStatus) then FStatus := nil;
end;

procedure TFileFinder.Search;
begin
	FStop := False;

	GetSubDirs(FRoot);
end;

procedure TFileFinder.Stop;
begin
	FStop := True;
end;

procedure TFileFinder.GetSubDirs(ARoot: String);
var
	Found: TSearchRec;
  Sub: String;
  i : Integer;
  Dirs: TStringList;
  Finished : Integer;
  iStop : Boolean;
begin
	iStop := FStop;
	Dirs := TStringList.Create;
	Finished := FindFirst(ARoot + '*.*', FFileAttr, Found);
  while (Finished = 0) and not (FStop or iStop) do
  begin
 		if (Found.Attr and faDirectory = faDirectory) and (Found.Name[1] <> '.') then
      	Dirs.Add(ARoot + Found.Name);
    if FFilename <> '' then
    	if FFuzzySearch then
      begin
      	if IsWild(Found.Name, FFilename, FIgnoreCase) then
    			if Assigned(FOnFound) then FOnFound(Self, ARoot, Found, iStop);
      end
      else
      begin
      	if FIgnoreCase then
        begin
        	if CompareText(Found.Name, FFilename)=0 then
    				if Assigned(FOnFound) then FOnFound(Self, ARoot, Found, iStop);
        end
        else
        begin
        	if CompareStr(Found.Name, FFilename)=0 then
    				if Assigned(FOnFound) then FOnFound(Self, ARoot, Found, iStop);
        end;
      end;
    FStop := iStop;
		Finished := FindNext(Found);
	  if Assigned(FStatus) then FStatus.Caption := ARoot;    
    if FProcessMsgs then Application.ProcessMessages;
  end;
  FindClose(Found);
	if FSubFolders and not (FStop or iStop) then
  	for i := 0 to Dirs.Count - 1 do
			GetSubDirs(Dirs[i] + '\');
  if ARoot = FRoot then
  	if Assigned(FOnStop) then FOnStop(Self);
  Dirs.Free;
end;

{--------------------------------------}
{ Register                             }
{--------------------------------------}
procedure Register;
begin
  RegisterComponents('Samples', [TFileFinder]);
end;

end.
